//
//  Commands.h
//  StreamSDKConLib
//
//  Copyright (c) 2014 StreamUnlimited. All rights reserved.
//

#ifndef __StreamSDKConLib__Commands__
#define __StreamSDKConLib__Commands__

#include "StreamAPI.h"
#include "Helpers.h"
#include "sdk_includes.h"

namespace Poco {
    class NotificationCenter;
}

namespace StreamUnlimited { namespace StreamAPI {
    class DeviceData {
        public:
            std::string name;
            std::string uuid;
            std::string ipAddress;
    };

    class Commands {
        private:
            StreamAPI _api;
            static const int _maxRedirects = 10;
            Poco::NotificationCenter  *_notificationCenter;
            bool getResultFromJson(const std::string &jsonResponse);
            Commands(const Commands&);
            Commands& operator=(const Commands&);

        public:
            explicit Commands(Poco::NotificationCenter *nc);
            ~Commands();

            StreamAPI *getApi(){ return &_api;}

            // root path
            std::string home();

            // network api
            bool connect(const std::string &host, Poco::UInt16 port = Poco::Net::HTTPSession::HTTP_PORT);
            bool disconnect(const bool &force);
            // network api

            // browsing
            int getRowsCount(const std::string &path);
            void getRowsRoles(const std::string &path, RolesValuesMap parentContainerData, std::string &redirectedPath, const int &startPosition, const int &count, int &totalRowsCount, RowsRolesVector &rows, std::string &parent_title);

            // activate only with path, or with path + data
            bool activate(const std::string &path, const std::string &data = "");

            // activate with path + type + value { "type": "string_", "string_": "online" }
            bool activate(const std::string &path, const std::string &type, const std::string &value);

            //In context menu activate items returns redirct, if exists
            bool activateInContextMenu(const std::string &path);
            bool cancel();
            // browsing

            //
            // Playback control
            //
            bool         play(const std::string &containerPath, RolesValuesMap rowToPlay, RolesValuesMap containerRoles);
            bool        pause();
            bool         stop();
            bool         next();
            bool     previous();
            bool         like();
            bool      dislike();

            // NOTE:
            // Not supported by vinjay codecs
            //
            /*
            bool   fastRewind();
            bool  fastForward();
            bool     stopScan();
            */

            bool setSeek2Time(const int &seconds);
            bool    setRepeat(const int &repeat);
            bool   setShuffle(const bool &random);
            bool      setMute(const bool &mute);
            bool    setVolume(const int &volume);
            bool setStandbyState(const RemoteAPI::StandbyState state);
#ifdef USE_NSDK_AMAZON_LOGIN
            bool amazonSetTokenBlob(const std::string &blob);
#else
            bool setAmazonLoginAppCode(const RemoteAPI::AmazonLoginAppCode &appCode);
#endif
            bool amazonLogout();

            bool setEnumValue(const std::string &path, const std::string &value);
            bool setTypedValue(const std::string &path, const std::string &type, const std::string &value);

            bool isGroupingSupported(const std::string &address, Poco::UInt16 port = Poco::Net::HTTPSession::HTTP_PORT);
            bool ping(const std::string &address, Poco::UInt16 port = Poco::Net::HTTPSession::HTTP_PORT);
            std::string getFriendlyName(const std::string &address, Poco::UInt16 port = Poco::Net::HTTPSession::HTTP_PORT);
            std::string setFriendlyName(const std::string &address, Poco::UInt16 port = Poco::Net::HTTPSession::HTTP_PORT, const std::string &name = "");
            std::string getUUID(const std::string &address, Poco::UInt16 port = Poco::Net::HTTPSession::HTTP_PORT);
            bool sendGroupingRequest(const std::string &address, Poco::UInt16 port, const DeviceData &deviceData, int groupingCommand);
            bool sendPersGroupingRequest(const std::string &address, Poco::UInt16 port, const std::string &groupName, std::vector<std::string> &receiversUUID);
            std::string getDevicesGroupMaster(const std::string& uuid, const std::string &address, Poco::UInt16 port = Poco::Net::HTTPSession::HTTP_PORT);
            std::vector<std::string> getSlaves(const std::string &address, Poco::UInt16 port = Poco::Net::HTTPSession::HTTP_PORT);

            int getNumItems(const std::string &path, std::string &redirectedPath, RolesValuesMap parentContainerData);
            void getSearchPath(const std::string &path, std::string &redirectedPath, std::vector<std::string> &titles, std::vector<std::string> &pathes);
            void browseSearchPath(const std::string &path, std::vector<std::string> &titles, std::vector<std::string> &pathes);
            RolesValuesMap getContainerData(const std::string &path);

            std::string setQuery(const std::string &path, const std::string &value);
            bool setValue(const std::string &path, const std::string &value);
            void postNotification(Poco::Notification::Ptr pNotification);
            std::string getRawValue(const std::string &path);

            // playlist
            bool createPlaylist(const std::string &name);
            bool deletePlaylist(int id);

            bool clearPlaylist(int id);
            bool setPlaylistName(int id, const std::string &name);
            bool setPlaylistShuffle(int id, RemoteAPI::RandomMode shuffle);
            bool setPlaylistRepeat(int id, RemoteAPI::RepeatMode repeat);

            bool movePlaylistItems(int id, std::vector<int> indicies, int to);
            bool addPlaylistItems(int id, RemoteAPI::AddItemMode addMode, std::vector<std::string> itemPaths);
            bool removePlaylistItems(int id, std::vector<int> indicies);

            // collections
            bool createCollection(const std::string &name);
            bool deleteCollection(int id);
            bool setCollectionName(int id, const std::string &name);
            bool addPlaylistToCollection(int collectionId, int playlistId);
            bool removePlaylistFromCollection(int collectionId, int playlistId);
    };
} }

#endif /* defined(__StreamSDKConLib__Commands__) */
