//
//  EventHandler.h
//  StreamSDKConLib
//
//  Copyright (c) 2014 StreamUnlimited. All rights reserved.
//

#ifndef __StreamSDKConLib__EventHandler__
#define __StreamSDKConLib__EventHandler__

#include <deque>
#include "StreamAPI.h"
#include "Notifications.h"

namespace Poco {
    class NotificationCenter;
    class Timer;
}

namespace StreamUnlimited { namespace StreamAPI {
    class EventHandler {
    private:
        Poco::NotificationCenter *_notificationCenter;
        StreamAPI           _api;
        std::string         _queueId;
        Poco::Mutex         _cs;
        bool                _pause;
        std::string         _host;
        Poco::UInt16        _port;

        std::deque<std::pair<std::string, std::string> > _scheduledSubscribeUnsubscribe;
        EventHandler(const EventHandler&);
        EventHandler& operator=(const EventHandler&);

    public:
        explicit EventHandler(Poco::NotificationCenter *nc);
        ~EventHandler(){}

        void connect(const std::string &host, Poco::UInt16 port);
        void disconnect();

        bool scheduleUpdateSubscription(const std::string &subscribe, const std::string &unsubscribe);
        bool clearScheduledUpdates();

        void updateSubscription(Poco::Timer &timer);
        void onTimer(Poco::Timer &timer);
        
        void pause();
        void unpause();

        void onVolumeChangedEvent();

        void onNotificationsChangedEvent();

        void onPlayerPlayDataEvent();

        void onAmazonLoggedInEvent();

        std::string getHost();
        Poco::UInt16 getPort();

    };
} }

#endif /* defined(__StreamSDKConLib__EventHandler__) */
