//
//  HTTPTransport.h
//  StreamSDKConLib
//
//  Copyright (c) 2014 StreamUnlimited. All rights reserved.
//

#ifndef __StreamSDKConLib__HTTPTransport__
#define __StreamSDKConLib__HTTPTransport__

#include <Poco/Net/HTTPClientSession.h>
#include <Poco/Mutex.h>

namespace StreamUnlimited { namespace StreamAPI {
    class  HTTPTransport {
        private:
            Poco::Mutex   _cs;
            Poco::Net::HTTPClientSession _session;
            std::string _host;
            Poco::UInt16 _port;
            bool         _connectionsAllowed;
            int          _timeoutCounter;

            // not safe to call from the outside
            void reset();
            HTTPTransport(const HTTPTransport&);
            HTTPTransport& operator=(const HTTPTransport&);

        public:
            HTTPTransport();
            HTTPTransport(const std::string &host, Poco::UInt16 port = Poco::Net::HTTPSession::HTTP_PORT);
            ~HTTPTransport();

            void setHost(const std::string &host, Poco::UInt16 port = Poco::Net::HTTPSession::HTTP_PORT);
            std::string getHost();
            Poco::UInt16 getPort();
            std::string sendRequest(const std::string &uri, const std::string &query, const int timeout = 30000, const int maxTimeoutRetries = 20);
            void disconnect();
            void cancelRequest();
            void setRequestTimeout(int milliseconds);
    };
} }

#endif /* defined(__StreamSDKConLib__HTTPTransport__) */
