//
//  Helpers.h
//  StreamSDKConLib
//
//  Copyright (c) 2014 StreamUnlimited. All rights reserved.
//

#ifndef __StreamSDKConLib__Helpers__
#define __StreamSDKConLib__Helpers__

#include <map>
#include <Poco/Any.h>
#include <Poco/JSON/Array.h>
#include "RemoteBrowser.h"

typedef std::map<std::string, Poco::Any> RolesValuesMap;
typedef std::vector<RolesValuesMap> RowsRolesVector;


namespace StreamUnlimited { namespace StreamAPI {

    class StreamAPI;

    std::string getStringFromMapSafe(const RolesValuesMap &map, const std::string &key);
    bool getBoolFromMapSafe(const RolesValuesMap &map, const std::string &key);

    /**
    *   Split string into vector using delimiters.
    */
    std::vector<std::string> explodeString(const std::string &string, const std::string &delimiters);

    /*
    *   Converts vector[rolesNames] and vector[rolesValues] into map[(roleNames, roleValues)]
    */
    RolesValuesMap mergeRolesNamesValues(const std::vector<std::string> &rolesNames, Poco::JSON::Array::Ptr rolesValues);


    int getRepeat(StreamAPI *api);
    bool getShuffle(StreamAPI *api);
    std::string calcPlayMode(int repeat, bool shuffle);
    RemoteAPI::StandbyState getStandby(StreamAPI *api);
    bool getMute(StreamAPI *api);
    bool getAmazonLoginDeviceInfo(StreamAPI *api, RemoteAPI::AmazonLoginDeviceInfo &info);
    bool setAmazonLoginAppCode(StreamAPI *api, const RemoteAPI::AmazonLoginAppCode &appCode);
    bool getAlexaVoiceServicePresent(StreamAPI *api);
#ifdef USE_NSDK_AMAZON_LOGIN
    std::string getAmazonCompanionUrl(StreamAPI *api);
    std::string getAmazonLoginBlob(StreamAPI *api);
#endif
    bool getAmazonLoggedIn(StreamAPI *api);
    RemoteAPI::VolumeStatus getVolume(StreamAPI *api);
    RemoteAPI::NotificationsStatus getNotifications(StreamAPI *api);
    int getPlayTime(StreamAPI *api);
    std::string getTitle(StreamAPI *api, std::string path);

    bool isRowBrowsable(RolesValuesMap row);
    bool isRowInvokable(RolesValuesMap row);
    bool isRowEditable(RolesValuesMap row);
    std::string getEditType(RolesValuesMap row);
    std::vector<RemoteAPI::EnumValue> getEnumValues(StreamAPI *api, RolesValuesMap row);
    std::string getValueType(RolesValuesMap row);
    std::string getRowType(RolesValuesMap row);
    RemoteAPI::ItemIcon getItemIcon(RolesValuesMap row);
    unsigned int getRowAttribute(RolesValuesMap row);
    std::vector<RemoteAPI::RowEntry> rowsRolesToRowEntries(StreamAPI *api, RowsRolesVector rows);

    bool checkIfSetPropperly(std::string val, StreamAPI *api, std::string path);
    const char* boolToString(bool value);
    std::string getTypedValueJson(const std::string &type, const std::string &value);
    RemoteAPI::MimeType parseMimeType(std::string mimeType);

    std::string escape(const std::string &inStr);

    bool isNonEmptyObject(const Poco::Dynamic::Var &var);
    bool isNonEmptyArray(const Poco::Dynamic::Var &var);
    bool isValidNonJsonResponse(const std::string &response);
    bool tryParseJson(const std::string &jsonstr, Poco::Dynamic::Var *object = NULL);
    bool tryAnyCast(const Poco::Any toConvert, std::string *value = NULL);

    bool mapContains(const RolesValuesMap &map, const std::string &key);
    std::string rolesMapToString(const RolesValuesMap &map);

    std::string escapeSpecialCharacters(const std::string &value);

} }
#endif /* defined(__StreamSDKConLib__Helpers__) */
