//
//  RemoteBrowser.h
//  StreamSDKConLib
//
//  Created by Ing. Stanislav Ruzani on 1/21/13.
//  Updated by DI Tereza Kovac Matic on 06/02/14.
//  Copyright (c) 2014 StreamUnlimited. All rights reserved.
//

/*! \mainpage StreamSDKConLib Library
 *
 * This document defines the Remote Browser interface to allow an external user interface to control the
 * Stream800 based streaming players over ethernet.
 *
 */

#ifndef __StreamSDKConLib__RemoteBrowser__
#define __StreamSDKConLib__RemoteBrowser__

#include <string>
#include <vector>

namespace StreamUnlimited { namespace RemoteAPI {
enum ConnectionState {
	eStateDisconnected = 0,
	eStateConnected    = 1
};

enum ViewType {
	eTypeUndefined  = 0,
	eTypeBrowse     = 1,
	eTypePlay       = 2,
	eTypeMessage    = 3,
	eTypeAlert      = 4,
	eTypeContextMenu= 5,
	eTypeTransition = 6
};

enum RepeatMode {
	eRepeatNone      = 0,
	eRepeatOne       = 1,
	eRepeatAll       = 2,
	eRepeatUndefined = 3
};

enum RandomMode {
	eRandomDisabled = 0,
	eRandomEnabled  = 1
};

enum PlayState {
	ePlayStateStop         = 0,
	ePlayStatePlay         = 1,
	ePlayStatePause        = 2,
	ePlayStateTransition   = 3,
	ePlayStateScanInc      = 4,
	ePlayStateScanDec      = 5,
	ePlayStateTrackStopped = 6,
	ePlayStateUndefined    = 7
};

enum RowEntryAttributes {
	eAttrUnknown    = 0x00,
	eAttrBrowsable  = 0x01,
	eAttrPlayable   = 0x02,
	eAttrEditable   = 0x04,
	eAttrInvokable  = 0x08,
	eAttrHeader     = 0x10,
	eAttrDisabled   = 0x20,
	eAttrCtxMenu    = 0x40,
	eAttrQuery      = 0x80
};

enum RowEditType {
	eEditTypeNone      = 0,
	eEditTypeString    = 1,
	eEditTypePassword  = 2,
	eEditTypeNumber    = 3,
	eEditTypeIPAddress = 4,
	eEditTypeExecute   = 5,
	eEditTypeRadioBox  = 6,
	eEditTypeCheckBox  = 7,
	eEditTypeSlider    = 8
};

enum ValidControls {
	ePlay              = 0x001,
	eStop              = 0x002,
	ePause             = 0x004,
	eNext              = 0x008,
	ePrevious          = 0x010,
	eScanInc           = 0x020,
	eScanDec           = 0x040,
	eSetRepeat         = 0x080,
	eSetRandom         = 0x100
};

enum AvailableControls {
    eControlNone       = 0x0000,
    eControlPause      = 0x0001,
    eControlNext       = 0x0002,
    eControlPrevious   = 0x0004,
    eControlSeekTime   = 0x0008,
    eControlSeekTrack  = 0x0010,
    eControlLike       = 0x0020,
    eControlDislike    = 0x0040,
    eControlShuffle    = 0x0080,
    eControlRepeatOne  = 0x0100,
    eControlRepeatAll  = 0x0200,
    eControlShuffleRepeatAll = 0x0400
};

enum ItemIcon {
	eItemIconNone               = 0,
	eItemIconFavorites          = 1,
	eItemIconIRadio             = 2,
	eItemIconFMRadio            = 3,
	eItemIconUSB                = 4,
	eItemIconIPod               = 5,
	eItemIconUPNP               = 6,
	eItemIconSamba              = 7,
	eItemIconTuneIn             = 8,
	eItemIconSirius             = 9,
	eItemIconSpotify            = 10,
	eItemIconPandora            = 11,
	eItemIconRhapsody           = 12,
	eItemIconNapster            = 13,
	eItemIconFlickr             = 14,
	eItemIconLineIn             = 15,
	eItemIconSettings           = 16,
	eItemIconHome               = 17,
	eItemIconFolder             = 18,
	eItemIconAudio              = 19,
	eItemIconVideo              = 20,
	eItemIconPicture            = 21,
	eItemIconHourglass          = 22,
	eItemIconCheckbox           = 23,
	eItemIconCount              = 24,
	eItemIconBluetooth          = 25,
	eItemIconAirPlay            = 26,
	eItemIconTidal              = 27,
	eItemIconPlaylists          = 28,
	eItemIconMultiroom          = 29,
	eItemIconRecentlyPlayed     = 30,
	eItemIconCDRom              = 31,
	eItemIconAirable            = 32,
	eItemIconAirableRadios      = 33,
	eItemIconAirableFeeds       = 34,
	eItemIconMusicLibrary       = 35,
	eItemIconQoboz              = 36,
	eItemIconDeezer             = 37,
	eItemIconHighResAudio       = 38
};

/*
enum SetRowsPlayMode {
	eSetRowsPlayModeStop      = 0,
	eSetRowsPlayModeContinue  = 1,
	eSetRowsPlayModePlayIndex = 2
};
*/

enum MimeType {
	eMimeType_None     = 0,
	eMimeType_Unknown  = 1,
	eMimeType_Mp3      = 2,
	eMimeType_Flac     = 3,
	eMimeType_Wma      = 4,
	eMimeType_Pcm      = 5,
	eMimeType_Aac      = 6,
	eMimeType_Mp4      = 7,
	eMimeType_Ogg      = 8,
	eMimeType_Wav      = 9,
	eMimeType_Aiff     = 10,
	eMimeType_Alac     = 11,
	eMimeType_Cnt      = 12,
	eMimeType_M3U      = 13,
	eMimeType_Dsd	   = 14
};

enum AddItemMode {
	eAddItemNow = 0,
	eAddItemAfterCurrent = 1,
	eAddItemAppend = 2
};

enum StandbyState {
	eStandbyStateUndefined = 0,
	eStandbyStateOnline = 10,
	eStandbyStateNetworkStandby = 15,
	eStandbyStateReboot = 20,
	eStandbyStateOffline = 30
};

/**
* A class that stores mediaData about items in a container
*/
class MediaData {
private:

public:
	//! Constructor
	MediaData()
	{
		clear();
	}

	//! Copy constructor
	MediaData(const MediaData &other)
	{
		*this = other;
	}

	//! Clear all stored attributes
	void clear()
	{
		_mimeTypeStr    = "";
		_mimeType       = eMimeType_None;
		_codec          = "";
		_playUri        = "";
		_title          = "";
		_artist         = "";
		_album          = "";
		_albumArtUri    = "";
	}

	MediaData &operator = (const MediaData &rhs)
	{
		if (this != &rhs)
		{
			_mimeTypeStr = rhs._mimeTypeStr;
			_mimeType    = rhs._mimeType;
			_codec       = rhs._codec;
			_playUri     = rhs._playUri;
			_title       = rhs._title;
			_artist      = rhs._artist;
			_album       = rhs._album;
			_albumArtUri = rhs._albumArtUri;
		}

		return *this;
	}

	std::string _mimeTypeStr;   /**< */
	MimeType    _mimeType;      /**< */
	std::string _codec;         /**< */
	std::string _playUri;       /**< */
	std::string _title;         /**< */
	std::string _artist;        /**< */
	std::string _album;         /**< */
	std::string _albumArtUri;   /**< */
};

/**
* A class that stores enums description
*/
class EnumValue {
private:

public:
	//! Constructor
	EnumValue()
	{
		clear();
	}

	//! Copy constructor
	EnumValue(const EnumValue &other)
	{
		*this = other;
	}

	//! Clear all stored attributes
	void clear()
	{
		_title      = "";
		_path       = "";
		_value      = "";
		_valueJson  = "";
	}

	EnumValue &operator = (const EnumValue &rhs)
	{
		if (this != &rhs)
		{
			_title      = rhs._title;
			_path       = rhs._path;
			_value      = rhs._value;
			_valueJson  = rhs._valueJson;
		}

		return *this;
	}

	std::string _title;     /**< Title of the enum value, should be rendered in UI */
	std::string _path;      /**< Path of the enum value usually in settings */
	std::string _value;     /**< Part of the _valueJson, this is what is stored in EditEnum._selected and should be compared against */
	std::string _valueJson; /**< The whole json encoded value */
};

/**
* A class that stores all possible enum values and selected value of an editable enum item
*/
class EditEnum {
private:

public:
	/// Constructor
	EditEnum()
	{
		clear();
	}

	/// Copy constructor
	EditEnum(const EditEnum &other)
	{
		*this = other;
	}

	/// Clear all stored attributes
	void clear()
	{
		_selected = "";
		_values.clear();
	}

	EditEnum &operator = (const EditEnum &rhs)
	{
		if (this != &rhs)
		{
			_selected = rhs._selected;
			_values   = rhs._values;
		}

		return *this;
	}

	std::string             _selected;  /**< Currentley selected value */
	std::vector<EnumValue>  _values;    /**< All possible values */
};

/**
* A class that stores description of an editable slider item
*/
class EditSlider {
private:

public:
	/// Constructor
	EditSlider()
	{
		clear();
	}

	/// Copy constructor
	EditSlider(const EditSlider &other)
	{
		*this = other;
	}

	/// Clear all stored attributes
	void clear()
	{
		_min = -1;
		_max = -1;
		_step = 0;
	}

	EditSlider &operator = (const EditSlider &rhs)
	{
		if (this != &rhs)
		{
			_min    = rhs._min;
			_max    = rhs._max;
			_step   = rhs._step;
		}

		return *this;
	}

	double _min;    /**< Minimum value of the slider */
	double _max;    /**< Maximum value of the slider */
	double _step;   /**< A increment step */
};

/**
* A class that stores description of an item in the view
*/
class RowEntry {
private:
public:
	//! Constructor
	RowEntry()
	{
		clear();
	}

	//! Copy constructor
	RowEntry(const RowEntry &other)
	{
		*this = other;
	}

	//! Clear all stored attributes
	void clear()
	{
		_name           = "";
		_description    = "";
		_editType       = eEditTypeNone;
		_editData       = "";
		_attrMask       = eAttrUnknown;
		_itemIcon       = eItemIconNone;
		_mediaData.clear();
		_editEnum.clear();
		_editSlider.clear();
	}

	RowEntry &operator = (const RowEntry &rhs)
	{
		if (this != &rhs)
		{
			_name        = rhs._name;
			_description = rhs._description;
			_attrMask    = rhs._attrMask;
			_editType    = rhs._editType;
			_editData    = rhs._editData;
			_editEnum    = rhs._editEnum;
			_editSlider  = rhs._editSlider;
			_itemIcon    = rhs._itemIcon;
			_mediaData   = rhs._mediaData;
		}

		return *this;
	}

	std::string     _name;          /**< Name of the row, to be rendered in UI */
	std::string     _description;   /**< More detailed row description if available */
	RowEditType     _editType;      /**< If the is not editable eEditTypeNone, if yes apropriate type */
	std::string     _editData;      /**< String representation of the current value of an editable item */
	EditEnum        _editEnum;      /**< If the item is ediable and of enum type, editType will be set to eEditTypeRadioBox and editEnum will store all possible values of the enum */
	EditSlider      _editSlider;    /**< If the item is editable and of slider type, editType will be set to eEditTypeSlider and editEnum will store the data needed for rendering/processing  */
	unsigned int    _attrMask;      /**< A bitwise OR of all apropriate row attributes defined in RowEntryAttributes */
	ItemIcon        _itemIcon;      /**< Icon type to be rendered */
	MediaData       _mediaData;     /**< Stores mediaData if the item is playable */
};

//! Volume status storing all relevant information about the volume
class VolumeStatus {
public:
    //! Constructor
    VolumeStatus()
    {
        clear();
    }

    void clear()
    {
        _currentVolume = 0;
        _minVolume     = 0;
        _maxVolume     = 100;
        _volumeStep    = 1;
    }

    int     _currentVolume;
    int     _minVolume;
    int     _maxVolume;
    int     _volumeStep;
};

//! Notifications status storing all relevant information about the notifications
class NotificationsStatus {
public:
    std::string _path;
    std::string _caption;
    std::vector<std::string> _text;

    //! Constructor
    NotificationsStatus()
    {
        clear();
    }

    void clear()
    {
        _path = "";
        _caption = "";
        _text.clear();
    }
};

// MqaInfo storing MQA relevant data
class MqaInfo
    {
    public:
        int         _outputSampleRate;
        int         _originalSampleRate;
        bool        _customRateUsed;
        std::string _originalSampleRateCustom;
        int         _creatorId;
        std::string _state;
        std::string _fullStringMqaInfo;

        void clear()
        {
            _outputSampleRate = 0;
            _originalSampleRate = 0;
            _customRateUsed = false;
            _originalSampleRateCustom = "";
            _creatorId = 0;
            _state = "";
            _fullStringMqaInfo = "";
        }

        MqaInfo()
        {
            clear();
        }

};

//! Play status storing all relevant information about the playback
class PlayStatus {
public:
	//! Constructor
	PlayStatus()
	{
		clear();
	}

	void clear()
	{
		_source             = "";
		_serviceName        = "";
		_title              = "";
		_mimeType           = eMimeType_None;
		_repeat             = eRepeatNone;
		_playState          = ePlayStateStop;
		_availableControls  = eControlNone;
		_random             = eRandomDisabled;

		_metaType           = "";
		_metaAlbumartUrl    = "";
		_metaArtist         = "";
		_metaAlbum          = "";
		_metaGenre          = "";
		_contextPath        = "";
		_liveStream         = false;
		_queueIndex         = 0;
		_queueLength        = 0;

		_instance           = -1;
		_playTime           = 0;
		_trackTime          = -1;
		_samplerate         = 0;
		_bitrate            = 0;
		_channels           = 0;
		_bps                = 0;
		_codec              = "";

	}

	std::string     _source;            /**<  */
	std::string     _serviceName;       /**<  */
	std::string     _title;             /**<  */
	MimeType        _mimeType;          /**<  */
	RepeatMode      _repeat;            /**<  */
	PlayState       _playState;         /**<  */
	RandomMode      _random;            /**<  */

	unsigned int    _availableControls; /**<  */
	int             _queueIndex;        /**<  */
	int             _queueLength;       /**<  */

	std::string     _metaType;          /**<  */
	std::string     _metaAlbumartUrl;   /**<  */
	std::string     _metaArtist;        /**<  */
	std::string     _metaAlbum;         /**<  */
	std::string     _metaGenre;         /**<  */
	std::string     _contextPath;       /**<  */
	bool            _liveStream;        /**<  */

	int             _playTime;          /**<  */
	int             _instance;          /**<  */
	int             _trackTime;         /**<  */
	int             _samplerate;        /**<  */
	int             _bitrate;           /**<  */
	int             _channels;          /**<  */
	int             _bps;               /**<  */
	std::string     _codec;             /**<  */
	MqaInfo      	_mqaInfo;           /**<  */
};

//! Message description
class Message {
private:
public:
	//! Construction
	Message()
	{
		clear();
	}

	//! Clear all stored attributes
	void clear()
	{
		_messageId      = "";
		_captionText    = "";
		_messageText    = "";
	}

	std::string _messageId;     /**<  Internaly used message identifier */
	std::string _captionText;   /**<  Title of the message */
	std::string _messageText;   /**<  More detailed text of the message */
};

//! Alert description
class Alert {
private:
public:
	//! Construction
	Alert()
	{
		clear();
	}

	//! Clear all stored attributes
	void clear()
	{
		_alertId        = "";
		_captionText    = "";
		_alertItems.clear();
	}

	std::string _alertId;                   /**<  Internaly used alert identifier */
	std::string _captionText;               /**<  Title of the message */
	std::vector<RowEntry> _alertItems;      /**<  More detailed text of the message */
};


//! ContextMenu description
class ContextMenu {
private:
public:
	//! Construction
	ContextMenu()
	{
		clear();
	}

	//! Clear all stored attributes
	void clear()
	{
		_title          = "";
		_numItems       = -1;
		_depth          = -1;
	}

	std::string _title;                  /**<  Internaly used alert identifier */
	int         _numItems;               /**<  Title of the message */
	long        _depth;
};

//! Amazon login device data
class AmazonLoginDeviceInfo {
private:
public:
    //! Construction
    AmazonLoginDeviceInfo()
    {
        clear();
    }

    void clear()
    {
        _serialNumber       = "";
        _productId          = "";
        _codeVerifierHash   = "";
        _scopeData          = "";
        _scope              = "";
    }

    std::string _serialNumber;
    std::string _productId;
    std::string _codeVerifierHash;
    std::string _scopeData;
    std::string _scope;
};

//! Amazon login return structure
class AmazonLoginAppCode {
private:
public:
    //! Construction
    AmazonLoginAppCode()
    {
        clear();
    }

    void clear()
    {
        _clientId       = "";
        _redirectUri    = "";
        _code           = "";
    }

    std::string _clientId;
    std::string _redirectUri;
    std::string _code;
};

/**
* Asynchronous notifications received from the device
*
* App developer should implement the RemoteBrowserObserver interface to get all relevant notifications from the S800 based device.
*/
class RemoteBrowserObserver {
private:
	// Interface classes should always be uncopyable
	RemoteBrowserObserver(const RemoteBrowserObserver&);
	RemoteBrowserObserver& operator=(const RemoteBrowserObserver&);

public:
	explicit RemoteBrowserObserver()
	{
		// dummy for Java
	}

	virtual ~RemoteBrowserObserver(){}

	/**
	* Called when the connection is closed for any reason
	*/
	virtual void onClientDisconnected() = 0;

	/**
	* Called when the view type changes
	*
	* View type is changed when a playback status changes (eTypePlay), an important message is received from the s800 based device (eTypeMessage),
	* and back to browsing view (eTypeBrowse). Idealy App developer should use the hook if needed to know when to switch between browse (list view), play screen and
	* some special message view. This is only a recomended view, app developer is not limited nor bound to render the view type suggested.
	* @param type Type of the view that should be rendered
	*/
	virtual void onViewTypeChanged(const ViewType &type) = 0;

	/**
	* Called whenever the current view changes
	*
	* Hook is invoked after browsing, and when the items are available. In the callback implementation app developer should call getItems @see RemoteBrowser::getItems().
	* It is also called when there is a change in the currentley viewed container.
	*/
	virtual void onViewChanged() = 0;

	/**
	 * Called whenever the current contextMenu changes
	 *
	 * It is called when there is a change in the currentley viewed container.
	 */
	virtual void onContextMenuViewChanged() = 0;

	/**
	* Called when the total count of items is changed/available
	*
	* After browsing and on the total number of items in the current view changes. Can be used for paging and to calculate the size of the list and how to render scroll-bars.
	* @param numItems Total count of items in the currentley viewed container
	*/
	virtual void onNumItemsChanged(const int &numItems) = 0;

	/**
	* Called when the play status changes
	*
	* @param status Status of the playback with all detailed infos
	*/
	virtual void onPlayStatusChanged(const PlayStatus &status) = 0;

	/**
	* Called when the play time changes
	*
	* @param time Current play time
	*/
	virtual void onPlayTimeChanged(const int &time) = 0;

	/**
	* Called when the volume changes
	*
	* @param volume Current volume status value
	*/
	virtual void onVolumeStatusChanged(const VolumeStatus &volumeStatus) = 0;


	/**
	* Called when the notifications changes
	*
	* @param volume Current volume status value
	*/
	virtual void onNotificationsStatusChanged(const NotificationsStatus &notificationsStatus) = 0;

	/**
	* Called when mute value changes
	*
	* @param mute Current mute value
	*/
	virtual void onMuteChanged(const bool &mute) = 0;

	/**
	* Called when "amazon logged in" property value changes
	*
	* @param mute Current "amazon logged in" value
	*/
	virtual void onAmazonLoggedInChanged(const bool &amazonLoggedIn) = 0;

	/**
	 * Called when mute value changes
	 *
	 * @param mute Current mute value
	 */
	virtual void onStandbyChanged(const StandbyState &standby) = 0;

	/**
	* Called when repeat mode changes
	*
	* @param repeat Current repeat value
	*/
	virtual void onRepeatChanged(const RepeatMode &repeat) = 0;

	/**
	* Called when shuffle mode changes
	*
	* @param shuffle Current shuffle value
	*/
	virtual void onShuffleChanged(const RandomMode &shuffle) = 0;

	/**
	* Called when data in tuneling node is changed. API is not mandatory and doesnt have to be overloaded in the observer.
	*/
	virtual void onTunnelDataChanged(const std::string &){}

	/**
	 * Called when a differen App should be opened
	 *
	 * @param app std::string
	 */
	virtual void onOpenApp(const std::string &app) = 0;

	/**
	 * Called to open a link in a browser of the controller
	 *
	 * @param link http link to be opened
	 */
	virtual void openLinkOnController(const std::string &link) = 0;
};

// convenience STL iterator macros
#ifdef _MSC_VER
#define SU_FOREACH(container, i)                 for (auto i = (container).begin(); i != (container).end(); ++i)
#define SU_FOREACH_REVERSE(container, i)         for (auto i = (container).rbegin(); i != (container).rend(); ++i)
#else
#define SU_FOREACH(container, i)                 for(typeof((container).begin()) i = (container).begin(); i != (container).end(); ++i)
#define SU_FOREACH_REVERSE(container, i)         for(typeof((container).rbegin()) i = (container).rbegin(); i != (container).rend(); ++i)
#endif

/**
 * Interface for remote browsing application
 *
 *
 */
class RemoteBrowser {
protected:
	RemoteBrowserObserver   *_observer; /**< Observer that gets notified whenever something changes in the browser */
	RemoteBrowser(RemoteBrowserObserver * const observer) : _observer(observer){}

public:
	virtual ~RemoteBrowser(){}

	/**
	* Factory for creating RemoteBrowser
	*/
	static RemoteBrowser *createRemoteBrowser(RemoteBrowserObserver * const observer);

	//
	// Connection management
	//

	/**
	* Pings S800 based device via IP address
	*
	* @param address IP address of the device to be pinged
	* @return [bool] success
	*/
	virtual bool ping(const std::string &address, int port) = 0;

	/**
	* Gets the name set by user of the device via IP address
	*
	* @param address IP address of the device of interest
	* @return [string] friendly device name
	*/
	virtual std::string getFriendlyName(const std::string &address, int port) = 0;

	/**
	* Sets the name set by user of the device via IP address
	*
	* @param address IP address of the device of interest
	* @param friendly name to be set
	* @return [string] friendly device name
	*/
	virtual std::string setFriendlyName(const std::string& ipAddress, int port, const std::string& name) = 0;

	/**
	* Connects to a remote S800 device
	*
	* @param address IP address of the device to connect
	* @return [bool] success
	*/
	virtual bool connect(const std::string &address, const int &port) = 0;

	/**
	* Disconnects from a remote S800 device
	*
	* @param force Flag to indicate if the disconnect should be forced or not
	* @param reset Flag to indicate if the session should be reset (remove all screen states, etc.)
	* @return [bool] success
	*/
	virtual bool disconnect(const bool &force, const bool &reset) = 0;

	/**
	* Asks for a full UI refresh. All the refreshed data will be provided via callback hooks.
	*
	* @return [bool] success
	*/
	virtual bool refresh() = 0;

	/**
	* Cancels a current network operation
	*
	* @return [bool] success
	*/
	virtual bool cancel() = 0;

	/**
	* Returns the current connection state
	*
	* @return StreamUnlimited::RemoteAPI::ConnectionState
	*/
	virtual ConnectionState const &getConnectionState() = 0;

	//
	// Browsing. All API calls are blocking!
	//

	/**
	* Gets the number of items in the current browsing view
	*
	* @return [int] total number of items
	*/
	virtual int getNumItems() = 0;

	/**
	* Gets the Volume level of the device
	*
	* @return [VolumeStatus] volume
	*/
	virtual VolumeStatus getVolume() = 0;

	/**
	* Gets depth of our view
	*
	* @return [int] current depth
	*/
	virtual int getCurrentDepth() = 0;

	/**
	 * Gets depth of our context menu view
	 *
	 * @return [int] current depth
	 */
	virtual int getCurrentContextMenuDepth() = 0;

	/**
	* Gets both the number of items in the current browsing view and its depth
	*
	* @param numItems Total number of items in the current view
	* @param depth Current browsing depth
	*/
	virtual void getNumItemsAndDepth(int &numItems, int &depth) = 0;

	/**
	* Gets items of the current view
	*
	* Operation is blocking and should be used with caution when fetching large ammount items.
	* It is advised configure the numItems to the max number of items that can be rendered on the screen.
	* Count of returned items will be <= numItems requested, depending on availability of items in the container.
	*
	* @param fromPosition Index of the starting item to be fetched
	* @param numItems Number of the items to fetch
	* @return [vector<RowEntry>] a vector of requested items represented via RowEntry
	*/
	virtual std::vector<RowEntry> getItems(const int &fromPosition, const int &numItems) = 0;

	/**
	 * Gets items of the current context menu view
	 *
	 * Operation is blocking and should be used with caution when fetching large ammount items.
	 * It is advised configure the numItems to the max number of items that can be rendered on the screen.
	 * Count of returned items will be <= numItems requested, depending on availability of items in the container.
	 *
	 * @param fromPosition Index of the starting item to be fetched
	 * @param numItems Number of the items to fetch
	 * @return [vector<RowEntry>] a vector of requested items represented via RowEntry
	 */
	virtual std::vector<RowEntry> getContextMenuItems(const int &fromPosition, const int &numItems) = 0;

	/**
	 * Get the searchTitles
	 *
	 * names for the search options
	 *
	 * @return returns current searchable option names
	 */
	virtual std::vector<std::string> getSearchTitles() = 0;

	/**
	 * search function
	 *
	 * @param searchString search string
	 * @param option id of the searchable option
	 */
	virtual void search(std::string searchString, int option=0) = 0;

	/**
	* Fetch items from the play queue
	*/
	// virtual std::vector<RowEntry> getPlayQueueItems(const int &fromPosition, const int &numItems) = 0;

	/**
	* set items to the play queue (for play queue only!)
	*/
	//virtual bool setPlayQueueItems(const SetRowsPlayMode &mode, const int &queueIndex, const std::vector<RowEntry> &rows) = 0;

	/**
	* Remove items from the play queue
	*/
	//virtual bool removePlayQueueItems(const int &startRow, const int &count) = 0;

	/**
	* Plays item at given position
	*
	* @param position Position of the item in the current view to be played
	* @return [bool] success
	*/
	virtual bool playItem(const int &position/*, const bool &fromQueue*/) = 0;

	/**
	* Plays item at given position in context menu
	*
	* @param position Position of the item in the current context view to be played
	* @return [bool] success
	*/
	virtual bool playContextMenuItem(const int &position/*, const bool &fromQueue*/) = 0;

	/**
	* Browse into item at given position
	*
	* @param position Position of the item in the current view to be browsed into
	* @return [bool] success
	*/
	virtual bool browseItem(const int &position) = 0;

	/**
	* Invoke query for given item
	*
	* @param position Position of the item in the current view to be invoked with query
	* @param queryStr String for which the query is executed
	* @return [bool] success
	*/
	virtual bool invokeQuery(const int &position, const std::string queryStr) = 0;

	/**
	* Gets the context menu of the item at the current position
	*
	* @param contextMenu pointer to a ContextMenu object, to copy to.
	* @return bool if succesfully fetched current ContextMenu
	*/
	virtual bool getContextMenu(ContextMenu &contextMenu) = 0;

	/**
	* closes the current ContextMenu and resets internal variables
	*
	* @return bool if succesful
	*/
	virtual bool closeContextMenu() = 0;

	/**
	* Browse context menu item at given position
	*
	* @param position Position of the item in the current view to be browsed into
	* @return [bool] success
	*/
	virtual bool browseContextMenuItem(const int &position) = 0;

	/**
	* Browse into context menu of an item at the given position
	*
	* If no context menu is opened, it will browse the context menu at the position of the
	* current browse view. If a context menu is currently open, then it will browse the
	* context menu of the context menu item at the given position
	*
	* @param position Position of the item in the current view to open the context menu of
	* @return [bool] success
	*/
	virtual bool browseIntoContextMenu(const int &position) = 0;

	/**
	* Browse into context menu at the given path
	*
	* If no context menu is opened, it will browse the context menu at the given path.
	*
	* @param path Path to context menu to be open
	* @return [bool] success
	*/
	virtual bool browseIntoContextMenu(const std::string &path) = 0;

	/**
	* Cancels the current browse operation
	*
	* @return [bool] success
	*/
	virtual bool browseCancel() = 0;

	/**
	 * Jumps to the parent view
	 *
	 * @return [bool] success
	 */
	virtual bool browseParent() = 0;

	/**
	 * Jumps to the parent view
	 *
	 * @return [bool] success
	 */
	virtual bool browseContextMenuParent() = 0;

	/**
	* Jumps to the top (home) view
	*
	* @return [bool] success
	*/
	virtual bool goHome() = 0;

	/**
	* Returns a name of the current view
	*
	* @return [string] title of the currentley viewed container
	*/
	virtual std::string getCurrentTitle() = 0;

	/**
	* Returns a name of the parent view
	*
	* @return [string] title of the currentley viewed container
	*/
	virtual std::string getParentTitle() = 0;

	/**
	* Gets the processing flag
	*
	* @return [bool] processing flag
	*/
	virtual bool isProcessing() = 0;

	/**
	* Gets the current view type
	*
	* @return [ViewType]
	*/
	virtual ViewType getViewType() = 0;

	/**
	* Change the current view type
	*
	* @return [bool] success
	*/
	virtual bool changeViewType(const ViewType &viewType) = 0;

	/**
	* Adds item to favourites list
	*
	* @return [bool] success
	*/
	virtual bool addToFavorites() = 0;

	/**
	* Removes item from favourites list
	*
	* @return [bool] success
	*/
	virtual bool removeFromFavorites(const int &row) = 0;

	//
	// Playback control
	//

	/**
	* Pauses the current playback
	*
	* @return [bool] success
	*/
	virtual bool pause() = 0;

	/**
	* Stops the current playback
	*
	* @return [bool] success
	*/
	virtual bool stop() = 0;

	/**
	* Jump to a next playback item. Playlogic is handled in the S800.
	*
	* @return [bool] success
	*/
	virtual bool next() = 0;

	/**
	* Jump to a previous playback item. Playlogic is handled in the S800.
	*
	* @return [bool] success
	*/
	virtual bool previous() = 0;

	virtual bool     like() = 0;
	virtual bool  dislike() = 0;

	/**
	* Switch to fast rewind mode
	*
	* @return [bool] success
	*/
	virtual bool fastRewind() = 0;

	/**
	* Switch to fast forward mode
	*
	* @return [bool] success
	*/
	virtual bool fastForward() = 0;

	/**
	* Stops scaning (back to the normal playback speed and direction)
	*
	* @return [bool] success
	*/
	virtual bool stopScan() = 0;

	/**
	* Jumps to a specific time
	*
	* @param seconds Position in seconds to which the playback should jump
	* @return [bool] success
	*/
	virtual bool setSeek2Time(const int &seconds) = 0;

	/**
	* Sets a desired repeat mode
	*
	* @param repeat Mode to be set
	* @return [bool] success
	*/
	virtual bool setRepeat(const RepeatMode &repeat) = 0;

	/**
	* Sets a desired random \(shuffle\) mode
	*
	* @param random Mode to be set
	* @return [bool] success
	*/
	virtual bool setShuffle(const RandomMode &random) = 0;

	/**
	* Toggles the mute value
	*
	* @param mute
	* @return [bool] success
	*/
	virtual bool setMute(const bool &mute) = 0;

	/**
	* Sets a desired repeat mode
	*
	* @param volume Volume level to be set in range [0; 100]
	* @return [bool] success
	*/
	virtual bool setVolume(const int &volume) = 0;

    /**
    * Checks if Alexa voice service is present
    *
    * @return [bool] true if AVS is present
    */
    virtual bool alexaVoiceServicePresent() = 0;

#ifdef USE_NSDK_AMAZON_LOGIN
	/**
	* Retrieve Amazon services companion URL
	*
	* @param url companion url will be stored here in case the call succeeds
	* @return [bool] success
	*/
	virtual bool amazonCompanionUrl(std::string &url) = 0;

    /**
    * Retrieve Amazon services login blob
    *
    * @param blob blob used for Amazon login
    * @return [bool] success
    */
    virtual bool amazonLoginBlob(std::string &blob) = 0;

    /**
    * Set token blob for Amazon services
    *
    * @param blob blob received from Amazon login process
    * @return [bool] success
    */
    virtual bool amazonSetTokenBlob(const std::string &blob) = 0;
#else

    /**
    * Retrieve device info required for Amazon App login
    *
    * @param info reference to structure where the info is stored
    * @return [bool] success
    */
    virtual bool amazonLoginDeviceInfo(RemoteAPI::AmazonLoginDeviceInfo &info) = 0;

    /**
    * Set Amazon login data
    *
    * @param appCode const reference to structure which contains Amazon login data
    * @return [bool] success
    */
    virtual bool amazonLoginAppCode(const RemoteAPI::AmazonLoginAppCode &appCode) = 0;
#endif

	/**
	* Returns true if user is logged into Amazon services
	*
	* @return [bool] true if we are logged into Amazon services
	*/
	virtual bool amazonLoggedIn() = 0;

	/**
	* Logout from Amazon services
	*
	* @return [bool] success
	*/
	virtual bool amazonLogout() = 0;

	//
	// Multiroom
	//

	/**
	* Checks if grouping/multiroom is supported on the device
	*
	* @param ipAddress IP address of the device
	* @return [bool] grouping supported flag
	*/
	virtual bool isGroupingSupported(const std::string &ipAddress, int port) = 0;

	/**
	* Group device on ipAddress with source device with sourceUUID
	*
	* @param ipAddress 			IP address of a slave/receiver device (provided via zeroconf - obtainable via discovery)
	* @param sourceUUID 		unique identifier of a master/source device (provided via zeroconf - obtainable via discovery)
	* @param sourceIpAddress 	IP address of a master/source device (provided via zeroconf - obtainable via discovery)
	* @param sourceName 		friendly name of a master/source device (provided via zeroconf - obtainable via discovery)
	* @return [bool] success
	*/
	virtual bool group(const std::string& ipAddress, int port, const std::string& sourceUUID, const std::string& sourceIpAddress, const std::string& sourceName) = 0;

	/**
	* Ungroup a follower with ipAddress
	*
	* @param ipAddress IP address of a slave/receiver device (provided via zeroconf - obtainable via discovery)
	* @return [bool] success
	*/
	virtual bool ungroup(const std::string& ipAddress, int port) = 0;

	/**
	* Creates persistent group on master devices with ipAddress and receivers UUID in receiversUUID
	*
	* @param ipAddress 			IP address of a master/source device (provided via zeroconf - obtainable via discovery)
	* @param groupName 			Name of group. Can be empty.
	* @param receiversUUID 		unique identifiers of a slave/receiver devices (provided via zeroconf - obtainable via discovery)
	* @return [bool] success
	*/
	virtual bool persGroups(const std::string& ipAddress, int port, const std::string& groupName, std::vector<std::string>& receiversUUID) = 0;

	/**
	* Returns UUID of devices master
	*
	* App developer can check to which group (master) a device blongs.
	*
	* @param address IP address of a device
	* @param uuid of a device
	* @return [string] UUID of devices master
	*/
	virtual std::string getGroup(const std::string& uuid, const std::string &ipAddress, int port) = 0;

	/**
	* Returns a vector of UUIDs of followers of the master given by address in argument
	*
	* App developer can check to which group (master) a device blongs.
	*
	* @param address IP address of a master device
	* @return [vector<string>] vector of UUIDs of slaves (followers)
	*/
	virtual std::vector<std::string> getSlaves(const std::string &address, int port) = 0;

	//
	// Message handling
	//

	/**
	* Retrieves current message
	*
	* @param message
	* @return [bool] success
	*/
	virtual bool getMessage(Message &message) = 0;

	/**
	* Confirms current message
	*
	* @param message
	* @return [bool] success
	*/
	virtual bool confirmMessage(const Message &message) = 0;

	/**
	* Cancels current message
	*
	* @param message
	* @return [bool] success
	*/
	virtual bool  cancelMessage(const Message &message) = 0;

	//
	// Alert handling
	//

	/**
	* Retrieves current message
	*
	* @param alert
	* @return [bool] success
	*/
	virtual bool getAlert(Alert &alert) = 0;

	/**
	* Confirms current message
	*
	* @param alert
	* @return [bool] success
	*/
	virtual bool closeAlert(const Alert &alert) = 0;

	//
	// Edit item
	//

	/**
	* Sets new value for editable item. Item is selected by its position in the view and.
	*
	* @param position Index of the items to be set
	* @param editData String representation of the value to be set.
	* @return [bool] success
	*/
	virtual bool setItem(const int &position, const std::string& editData) = 0;

	/**
	* Sets new value for editable item in context menu. Item is selected by its position in the view and.
	*
	* @param position Index of the items to be set
	* @param editData String representation of the value to be set.
	* @return [bool] success
	*/
	virtual bool setContextMenuItem(const int &position, const std::string& editData) = 0;

	//
	// Direct access API (no iteration necessary)
	//

	/**
	* Browse into item at given path. After direct browsing its possible to continue sequential browsing by index in the container.
	*
	* @param path Item path for direct browsing
	* @param title Title of the new browse stack
	* @param startPos Index of the starting item to be fetched
	* @param count Number of the items to fetch
	* @return [bool] success
	*/
	virtual bool browse(const std::string &path, const std::string &title, const int &startPos, const int &count, const bool direct = true) = 0;

	/**
	* Get value of an item at given path
	*
	* @param path Item path
	* @return [string] JSON encoded value of the item; directely provided by s800
	*/
	virtual std::string get(const std::string &path) = 0;

	/**
	* Activate item at given path
	*
	* @param path Item path
	* @param data Data (json data)
	* @return [bool] success
	*/
	virtual bool activate(const std::string &path, const std::string &data = "") = 0;

	/**
	* Activate item at given path
	*
	* @param path Item path
	* @param type NsdkType in string format ( like: string_ )
	* @param value Value in string format
	* @return [bool] success
	*/
	virtual bool activate(const std::string &path, const std::string &type, const std::string &value) = 0;

	/**
	* Activate item at given position
	*
	* @param path Item path
	* @param data Data (json data)
	* @return [bool] success
	*/
	virtual bool activateItem(int position, const std::string &data = "") = 0;

	/**
	* Activate item at given position
	*
	* @param position
	* @param type NsdkType in string format ( like: string_ )
	* @param value Value in string format
	* @return [bool] success
	*/
	virtual bool activateItem(int position, const std::string &type, const std::string &value) = 0;

	/**
	 * Activate the power state nodes
	 *
	 * @param state enum type
	 * @return [bool] success
	 */
	virtual bool setStandbyState(const StandbyState state) = 0;

	/**
	* Set new value for value items
	*
	* API is to be used with great caution and deep knowledge of the SDK since the value is directley set to a specific path in a S800 device.
	*
	* @param path Item path
	* @param value JSON encoded new value of the item to be set.
	* @return [bool] success
	*/
	virtual bool set(const std::string &path, const std::string& value) = 0;

	// Browsing with Anchor

	/**
	* Browse into item at given path. After direct browsing its possible to continue sequential browsing by index in the container.
	* Save Anchor which can be used to jump back to that position in the future.
	*
	* @param path Item path for direct browsing
	* @param fromPosition Index of the starting item to be fetched
	* @param numItems Number of the items to fetch
	* @return [bool] success
	*/
	virtual bool browseAndSetAnchor(const std::string &path, const int &startPos, const int &count) = 0;

	/**
	* Browse into item at given position and save Anchor which can be used to jump back to that position in the future.
	*
	* @param position Position of the item in the current view to be browsed into
	* @return [bool] success
	*/
	virtual bool browseItemAndSetAnchor(const int &position) = 0;

	/**
	* Browse into Anchor which is previously saved with browseAndSetAnchor() or browseItemAndSetAnchor().
	*
	* @param position Position of the item in the current view to be browsed into
	* @return [bool] success
	*/
	virtual bool browseAnchor() = 0;

	/**
	* Browse into item at given path, beginning from anchor. After direct browsing its possible to continue sequential browsing by index in the container.
	*
	* @param path Item path for direct browsing
	* @param startPosition Index of the starting item to be fetched
	* @param count Number of the items to fetch
	* @return [bool] success
	*/
	virtual bool browseFromAnchor(const std::string &path, const int &startPos, const int &count) = 0;

	// Playlist handling

	/**
	* Browse all playlists
	*
	* @return [bool] success
	*/
	virtual bool browsePlaylists() = 0;

	/**
	* Browse the playqueue
	*
	* @return [bool] success
	*/
	virtual bool browsePlayqueue() = 0;

	/**
	* Create a new playlist with the given name
	*
	* @param name playlist name
	* @return [bool] success
	*/
	virtual bool createPlaylist(const std::string &name) = 0;

	/**
	* Delete a playlist which has the id 'id'
	*
	* @param id playlist id
	* @return [bool] success
	*/
	virtual bool deletePlaylist(int id) = 0;

	/**
	* Remove all the tracks from a playlist which has the id 'id'
	*
	* @param id playlist id
	* @return [bool] success
	*/
	virtual bool clearPlaylist(int id) = 0;

	/**
	* Set the playlist name
	*
	* @param name playlist name
	* @return [bool] success
	*/
	virtual bool setPlaylistName(int id, const std::string &name) = 0;

	/**
	* Set the shuffle mode fot the specified playlist
	*
	* @param id playlist id
	* @param shuffle shuffle mode (0: OFF, 1: ON)
	* @return [bool] success
	*/
	virtual bool setPlaylistShuffle(int id, RandomMode shuffle) = 0;

	/**
	* Set the repeat mode fot the specified playlist
	*
	* @param id playlist id
	* @param repeat repeat mode (0: None, 1: One, 2: All, 3: Undefined)
	* @return [bool] success
	*/
	virtual bool setPlaylistRepeat(int id, RepeatMode repeat) = 0;

	/**
	* Move an item in the currently browsed playlist
	*
	* @param from the index (sequence number) of the item
	* @param to the new index-position (sequence number) of the item
	* @return [bool] success
	*/
	virtual bool movePlaylistItems(std::vector<int> indicies, int to) = 0;

	/**
	* Add items to the specified playlist
	*
	* @param id playlist id
	* @param addMode the mode of adding new item (0: Now, 1: AfterCurrent, 2: Append)
	* @param itemPaths vector of items (paths) to be added
	* @return [bool] success
	*/
	virtual bool addPlaylistItems(int id, AddItemMode addMode, std::vector<std::string> itemPaths) = 0;

	/**
	* Remove items from the specified playlist
	*
	* @param id playlist id
	* @param indicies vector of items (indicies) to be removed
	* @return [bool] success
	*/
	virtual bool removePlaylistItems(int id, std::vector<int> indicies) = 0;

	/**
	* Create a new collection with the given name
	*
	* @param name collection name
	* @return [bool] success
	*/
	virtual bool createCollection(const std::string &name) = 0;

	/**
	* Delete a collection which has the id 'id'
	*
	* @param id collection id
	* @return [bool] success
	*/
	virtual bool deleteCollection(int id) = 0;

	/**
	* Set the collection name
	*
	* @param name collection name
	* @return [bool] success
	*/
	virtual bool setCollectionName(int id, const std::string &name) = 0;

	/**
	* Add an existing playlist to an existing collection
	*
	* @param collectionId collection id
	* @param playlistId playlist id
	* @return [bool] success
	*/
	virtual bool addPlaylistToCollection(int collectionId, int playlistId) = 0;

	/**
	* Remove an existing playlist from an existing collection
	*
	* @param collectionId collection id
	* @param playlistId playlist id
	* @return [bool] success
	*/
	virtual bool removePlaylistFromCollection(int collectionId, int playlistId) = 0;

};
} }

#endif /* defined(__StreamSDKConLib__RemoteBrowser__) */
