//
//  ScreenStack.h
//  StreamSDKConLib
//
//  Copyright (c) 2016 StreamUnlimited. All rights reserved.
//

#ifndef __StreamSDKConLib__Screen__
#define __StreamSDKConLib__Screen__

#include <string>
#include <vector>
#include "Helpers.h"

namespace StreamUnlimited {
    namespace StreamAPI {
        class EventPoller;

        class Screen {
        public:
            std::string _path;
            std::string _title;
            RolesValuesMap _parentContainerData;
            bool _direct;

            Screen(const std::string& path, const std::string& title,
                   const RolesValuesMap& parentContainerData)
                : _path(path)
                , _title(title)
                , _parentContainerData(parentContainerData)
                , _direct(false)
            {
            }
        };

        class ScreenStack {
        public:
            ScreenStack(EventPoller& poller)
                : _eventPoller(poller)
            {
            }
            void pushScreen(const Screen&);
            void pushDirectScreen(const Screen&);
            void popScreen();
            void closeAllScreens(unsigned int keep = 0);
            /// This method erases paths that start with the string
            /// supplied, eraseWithPath("ui:/airable") will close the
            /// first path that has that prefix such as
            /// "ui:/airable/home".
            bool eraseWithPath(const std::string&);
            Screen& back() {return _stack.back();}
            size_t size() const {return _stack.size();}
            Screen& at(const size_t i) {return _stack.at(i);}

        private:
            EventPoller& _eventPoller;
            std::vector<Screen> _stack;
        };
    }
}

#endif /* __StreamSDKConLib__Screen__ */
