//
//  StreamAPI.h
//  StreamSDKConLib
//
//  Copyright (c) 2014 StreamUnlimited. All rights reserved.
//

#ifndef __StreamSDKConLib__StreamAPI__
#define __StreamSDKConLib__StreamAPI__

#include <Poco/NotificationCenter.h>
#include "HTTPTransport.h"

namespace StreamUnlimited { namespace StreamAPI {
    enum Command {
        eGetData = 0,
        eSetData = 1,
        eGetRows = 2
    };

    class  StreamAPI {
        private:
            HTTPTransport _httpTransport;
            Poco::NotificationCenter *_notificationCenter;
            StreamAPI(const StreamAPI&);
            StreamAPI& operator=(const StreamAPI&);

        public:
            explicit StreamAPI(Poco::NotificationCenter *nc);
            ~StreamAPI();

            void connect(const std::string &host, Poco::UInt16 port = Poco::Net::HTTPSession::HTTP_PORT);
            void disconnect();

            // Low-level stream api calls
            std::string setData(const std::string &path, const std::string &roles, const std::string &value);
            Poco::Dynamic::Var getData(const std::string &path, const std::string &roles);
            Poco::Dynamic::Var getRows(const std::string &path, const std::string &roles, int startRow, int endRow);

            // Low-level event functions - blocking functions
            std::string modifyQueue(const std::string &queueId, const std::string &subscribePath, const std::string &unsubscribePath);
            std::string pollQueue(const std::string &queueId, int timeout);

            bool cancel();

            std::string directHTTPRequest(const std::string &address, Poco::UInt16 port, const Command &command, const std::string &path, const std::string &roles, const std::string &value, const int maxTimeoutRetries = 1);
            std::string getRawData(const std::string &path, const std::string &roles);

        private:
            void postNotification(Poco::Notification::Ptr pNotification);
            std::string doEncoding(const std::string &value, const std::string &reserved);
            bool isMethodNotImplementedError(const std::string &error);
    };
} }

#endif /* defined(__StreamSDKConLib__StreamAPI__) */
